"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("../index");
class MemoryLogger extends index_1.Logger {
    constructor() {
        super(...arguments);
        this._logs = [];
    }
    normalizeParams() {
        super.normalizeParams();
        this._maxItems = this._params.maxItems || 1000;
    }
    _log(level, ...args) {
        if (this._logs.length >= this._maxItems) {
            this._logs.shift();
        }
        this._logs.push({
            level: level,
            args: args
        });
    }
    getLogs() {
        return this._logs;
    }
    static getModda() {
        return {
            uuid: "Webda/MemoryLogger",
            label: "MemoryLogger",
            description: "Keep everything in memory",
            webcomponents: [],
            logo: "images/icons/none.png",
            configuration: {
                schema: {
                    type: "object",
                    properties: {
                        maxItems: {
                            type: "number",
                            value: 1000
                        },
                        logLevel: {
                            type: "string",
                            value: "INFO"
                        },
                        logLevels: {
                            type: "string",
                            value: "ERROR,WARN,CONSOLE,INFO,DEBUG"
                        }
                    }
                }
            }
        };
    }
}
exports.MemoryLogger = MemoryLogger;
//# sourceMappingURL=memorylogger.js.map