"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const uuid = require("uuid");
const index_1 = require("../index");
class CoreModelMapper {
    constructor(uuid, properties = {}, store) {
        this.uuid = uuid;
    }
    load() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.__store.get(this.uuid);
        });
    }
}
/**
 * First basic model for Ident
 * Will evolve with version 0.2 and Model addition
 *
 * @class
 */
class CoreModel extends index_1.OwnerPolicy {
    static getActions() {
        return {};
    }
    getAvailableActions() {
        return {};
    }
    load(raw, secure = false) {
        if (!raw) {
            return;
        }
        if (!raw.uuid) {
            raw.uuid = this.generateUid(raw);
        }
        for (let prop in raw) {
            if (!secure && prop[0] === "_") {
                continue;
            }
            this[prop] = raw[prop];
        }
    }
    /**
     * Return the object registered store
     */
    getStore() {
        return this.__store;
    }
    /**
     * Get the object again
     *
     * @throws Error if the object is not coming from a store
     */
    refresh() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.__store) {
                throw Error("No store linked to this object");
            }
            let obj = yield this.__store.get(this.uuid);
            if (obj) {
                for (let i in obj) {
                    this[i] = obj[i];
                }
                for (let i in this) {
                    // @ts-ignore
                    if (obj[i] !== this[i]) {
                        this[i] = undefined;
                    }
                }
            }
            return this;
        });
    }
    /**
     * Delete this object
     *
     * @throws Error if the object is not coming from a store
     */
    delete() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.__store) {
                throw Error("No store linked to this object");
            }
            return this.__store.delete(this.uuid);
        });
    }
    /**
     * Save this object
     *
     * @throws Error if the object is not coming from a store
     */
    save() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.__store) {
                throw Error("No store linked to this object");
            }
            let obj = yield this.__store.save(this);
            for (var i in obj) {
                this[i] = obj[i];
            }
        });
    }
    /**
     * Update this object
     *
     * @throws Error if the object is not coming from a store
     */
    update(changes) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.__store) {
                throw Error("No store linked to this object");
            }
            let obj = yield this.__store.update(changes, this.uuid);
            for (var i in obj) {
                this[i] = obj[i];
            }
        });
    }
    /**
     * Return the object schema, if defined any modification done to the object by external source
     * must comply to this schema
     */
    _getSchema() {
        return;
    }
    validate(ctx, updates = undefined) {
        return __awaiter(this, void 0, void 0, function* () {
            let schema = this._getSchema();
            if (!schema) {
                return true;
            }
            if (updates) {
                this.load(updates);
            }
            if (!ctx._webda.validate(this, schema)) {
                throw Error(ctx._webda.validationLastErrors());
            }
            return true;
        });
    }
    generateUid(object = undefined) {
        return uuid.v4().toString();
    }
    _jsonFilter(key, value) {
        if (key[0] === "_" && key.length > 1 && key[1] === "_") {
            return undefined;
        }
        return value;
    }
    toStoredJSON(stringify = false) {
        let obj = this._toJSON(true);
        obj.__store = undefined;
        if (stringify) {
            return JSON.stringify(obj);
        }
        return obj;
    }
    _getService(service) {
        if (!this.__store) {
            return undefined;
        }
        return this.__store.getService(service);
    }
    _toJSON(secure) {
        let obj = {};
        for (let i in this) {
            let value = this[i];
            if (!secure) {
                value = this._jsonFilter(i, this[i]);
            }
            if (value === undefined)
                continue;
            obj[i] = value;
        }
        return obj;
    }
    toJSON() {
        return this._toJSON(false);
    }
    _onDelete() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    _onDeleted() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    _onGet() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    _onSave() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    _onSaved() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    _onUpdate(updates) {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    _onUpdated() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
}
exports.CoreModel = CoreModel;
//# sourceMappingURL=coremodel.js.map