"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
class OwnerPolicy {
    /**
     * Return false if can't create
     */
    canCreate(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            this._user = ctx.getSession().getUserId();
            if (!this._user) {
                throw 403;
            }
            return this;
        });
    }
    getOwner() {
        return this._user;
    }
    canAct(ctx, action) {
        return __awaiter(this, void 0, void 0, function* () {
            if (action === "create") {
                return this.canCreate(ctx);
            }
            else if (action === "update" ||
                action === "attach_binary" ||
                action === "detach_binary") {
                return this.canUpdate(ctx);
            }
            else if (action === "get" || action === "get_binary") {
                return this.canGet(ctx);
            }
            else if (action === "delete") {
                return this.canDelete(ctx);
            }
            throw 403;
        });
    }
    /**
     * Return false if can't update
     */
    canUpdate(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            // Allow to modify itself by default
            if ((!this.getOwner() || ctx.getSession().getUserId() !== this.getOwner()) &&
                ctx.getSession().getUserId() !== this.uuid) {
                throw 403;
            }
            return this;
        });
    }
    /**
     * Return false if can't get
     */
    canGet(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.public) {
                return this;
            }
            return this.canUpdate(ctx);
        });
    }
    /**
     * Return false if can't delete
     */
    canDelete(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.canUpdate(ctx);
        });
    }
}
exports.OwnerPolicy = OwnerPolicy;
//# sourceMappingURL=ownerpolicy.js.map