"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const mocha_typescript_1 = require("mocha-typescript");
const assert = require("assert");
const index_1 = require("../index");
const test_1 = require("../test");
var config = require("../../test/config.json");
const Utils = require("../../test/utils");
let OwnerPolicyTest = class OwnerPolicyTest extends test_1.WebdaTest {
    before() {
        const _super = Object.create(null, {
            before: { get: () => super.before }
        });
        return __awaiter(this, void 0, void 0, function* () {
            yield _super.before.call(this);
            this._ctx = yield this.webda.newContext(new index_1.HttpContext("test.webda.io", "GET", "/"));
            this._session = this._ctx.newSession();
            this._session.login("none", "none");
            this._ctx.getCurrentUser = () => __awaiter(this, void 0, void 0, function* () {
                return this._user;
            });
            this._taskStore = this.webda.getService("Tasks");
            this._userStore = this.webda.getService("Users");
            assert.notEqual(this._taskStore, undefined);
            assert.notEqual(this._userStore, undefined);
        });
    }
    beforeEach() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._taskStore.__clean();
            yield this._userStore.__clean();
            this._ctx = yield this.newContext({});
            yield this._taskStore.save({
                _user: "fake_user",
                uuid: "task_user1",
                name: "for_schema"
            });
            yield this._taskStore.save({
                _user: "fake_user2",
                uuid: "task_user2"
            });
            yield this._taskStore.save({
                _user: "fake_user3",
                uuid: "task_public",
                public: true
            });
        });
    }
    postNotLogged() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.beforeEach();
            let executor = this.getExecutor(this._ctx, "test.webda.io", "POST", "/tasks", {
                name: "Task #1"
            });
            assert.notEqual(executor, undefined);
            yield Utils.throws(executor.execute.bind(executor, this._ctx), res => res == 403);
        });
    }
    post() {
        return __awaiter(this, void 0, void 0, function* () {
            let executor = this.getExecutor(this._ctx, "test.webda.io", "POST", "/tasks", {
                name: "Task #1"
            });
            this._ctx.session.login("fake_user", "fake_ident");
            yield executor.execute(this._ctx);
            let task = JSON.parse(this._ctx.getResponseBody());
            assert.equal(task.name, "Task #1");
        });
    }
    getWrongOwner() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.beforeEach();
            this._ctx.getSession().login("fake_user2", "fake_ident");
            let executor = this.getExecutor(this._ctx, "test.webda.io", "GET", "/tasks/task_user1", {});
            yield Utils.throws(executor.execute.bind(executor, this._ctx), res => res == 403);
        });
    }
    putWrongOwner() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.beforeEach();
            this._ctx.session.login("fake_user2", "fake_ident");
            let executor = this.getExecutor(this._ctx, "test.webda.io", "PUT", "/tasks/task_user1", {});
            yield Utils.throws(executor.execute.bind(executor, this._ctx), res => res == 403);
        });
    }
    put() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.beforeEach();
            this._ctx.session.login("fake_user", "fake_ident");
            let executor = this.getExecutor(this._ctx, "test.webda.io", "PUT", "/tasks/task_user1", { public: true });
            yield executor.execute(this._ctx);
            let result = JSON.parse(this._ctx.getResponseBody());
            assert.equal(result.uuid, "task_user1");
            assert.equal(result.public, true);
        });
    }
    getPublic() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.beforeEach();
            this._ctx.session.login("fake_user2", "fake_ident");
            let executor = this.getExecutor(this._ctx, "test.webda.io", "GET", "/tasks/task_public");
            yield executor.execute(this._ctx);
            let result = JSON.parse(this._ctx.getResponseBody());
            assert.equal(result.uuid, "task_public");
        });
    }
    actioms() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.beforeEach();
            this._ctx.session.login("fake_user2", "fake_ident");
            let executor = this.getExecutor(this._ctx, "test.webda.io", "GET", "/tasks/task_user1/actionable");
            yield executor.execute(this._ctx);
            executor = this.getExecutor(this._ctx, "test.webda.io", "PUT", "/tasks/task_user1/impossible");
            yield Utils.throws(executor.execute.bind(executor, this._ctx), res => res == 403);
        });
    }
    delete() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.beforeEach();
            this._ctx.session.login("fake_user2", "fake_ident");
            let executor = this.getExecutor(this._ctx, "test.webda.io", "DELETE", "/tasks/task_user2");
            yield executor.execute(this._ctx);
        });
    }
    deleteWrongOwner() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.beforeEach();
            this._ctx.session.login("fake_user2", "fake_ident");
            let executor = this.getExecutor(this._ctx, "test.webda.io", "DELETE", "/tasks/task_user1");
            yield Utils.throws(executor.execute.bind(executor, this._ctx), res => res == 403);
        });
    }
    deleteWrongOwnerPublic() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.beforeEach();
            this._ctx.session.login("fake_user2", "fake_ident");
            let executor = this.getExecutor(this._ctx, "test.webda.io", "DELETE", "/tasks/task_public");
            yield Utils.throws(executor.execute.bind(executor, this._ctx), res => res == 403);
        });
    }
    deleteUnknown() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.beforeEach();
            this._ctx.session.login("fake_user2", "fake_ident");
            let executor = this.getExecutor(this._ctx, "test.webda.io", "DELETE", "/tasks/task_unknown");
            yield Utils.throws(executor.execute.bind(executor, this._ctx), res => res == 404);
        });
    }
};
__decorate([
    mocha_typescript_1.test("POST - not logged")
], OwnerPolicyTest.prototype, "postNotLogged", null);
__decorate([
    mocha_typescript_1.test("POST")
], OwnerPolicyTest.prototype, "post", null);
__decorate([
    mocha_typescript_1.test("GET - wrong owner")
], OwnerPolicyTest.prototype, "getWrongOwner", null);
__decorate([
    mocha_typescript_1.test("PUT - wrong owner")
], OwnerPolicyTest.prototype, "putWrongOwner", null);
__decorate([
    mocha_typescript_1.test("PUT")
], OwnerPolicyTest.prototype, "put", null);
__decorate([
    mocha_typescript_1.test("GET - public")
], OwnerPolicyTest.prototype, "getPublic", null);
__decorate([
    mocha_typescript_1.test("Actions")
], OwnerPolicyTest.prototype, "actioms", null);
__decorate([
    mocha_typescript_1.test("DELETE")
], OwnerPolicyTest.prototype, "delete", null);
__decorate([
    mocha_typescript_1.test("DELETE - wrong owner")
], OwnerPolicyTest.prototype, "deleteWrongOwner", null);
__decorate([
    mocha_typescript_1.test("DELETE - wrong owner - public")
], OwnerPolicyTest.prototype, "deleteWrongOwnerPublic", null);
__decorate([
    mocha_typescript_1.test("DELETE - unknown")
], OwnerPolicyTest.prototype, "deleteUnknown", null);
OwnerPolicyTest = __decorate([
    mocha_typescript_1.suite
], OwnerPolicyTest);
//# sourceMappingURL=ownerpolicy.spec.js.map