"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("../index");
const uuid = require("uuid");
class MemoryQueue extends index_1.Queue {
    init() {
        const _super = Object.create(null, {
            init: { get: () => super.init }
        });
        return __awaiter(this, void 0, void 0, function* () {
            yield _super.init.call(this);
            this._queue = {};
            if (!this._params.expire) {
                this._params.expire = 30;
            }
            this._params.expire *= 1000;
        });
    }
    size() {
        return __awaiter(this, void 0, void 0, function* () {
            return Object.keys(this._queue).length;
        });
    }
    sendMessage(params) {
        return __awaiter(this, void 0, void 0, function* () {
            var uid = uuid.v4();
            // Avoid duplication
            while (this._queue[uid]) {
                uid = uuid.v4();
            }
            this._queue[uid] = {
                Body: params,
                Claimed: 0,
                ReceiptHandle: uid
            };
        });
    }
    receiveMessage() {
        return __awaiter(this, void 0, void 0, function* () {
            for (var i in this._queue) {
                if (this._queue[i].Claimed < new Date().getTime() - this._params.expire) {
                    this._queue[i].Claimed = new Date().getTime();
                    return [this._queue[i]];
                }
            }
            return [];
        });
    }
    deleteMessage(receipt) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._queue[receipt]) {
                delete this._queue[receipt];
            }
        });
    }
    __clean() {
        return __awaiter(this, void 0, void 0, function* () {
            this._queue = {};
        });
    }
    static getModda() {
        return {
            uuid: "Webda/MemoryQueue",
            label: "MemoryQueue",
            description: "Implements a simple in memory queue",
            webcomponents: [],
            documentation: "",
            logo: "images/icons/memoryqueue.png",
            configuration: {
                default: {}
            }
        };
    }
}
exports.MemoryQueue = MemoryQueue;
//# sourceMappingURL=memoryqueue.js.map