"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const queue_spec_1 = require("./queue.spec");
const index_1 = require("../index");
const mocha_typescript_1 = require("mocha-typescript");
const assert = require("assert");
let MemoryQueueTest = class MemoryQueueTest extends queue_spec_1.QueueTest {
    abstractQueue() {
        // Ensure abstract - mainly for code coverage
        let queue = new index_1.Queue(undefined, undefined, undefined);
        this.assertThrowsAsync(queue.sendMessage, Error);
        this.assertThrowsAsync(queue.size, Error);
        this.assertThrowsAsync(queue.receiveMessage, Error);
        this.assertThrowsAsync(queue.deleteMessage, Error);
    }
    worker() {
        return __awaiter(this, void 0, void 0, function* () {
            yield new Promise(resolve => {
                let queue = new index_1.Queue(undefined, undefined, undefined);
                let seq = 0;
                queue._webda = {
                    log: () => { }
                };
                queue.receiveMessage = () => {
                    seq++;
                    switch (seq) {
                        case 1:
                            // Test the resume if no messages available
                            return Promise.resolve([]);
                        case 2:
                            return Promise.resolve([
                                {
                                    ReceiptHandle: "msg1",
                                    Body: '{"title":"plop"}'
                                }
                            ]);
                        case 3:
                            throw Error();
                        case 4:
                            // An error occured it should double the pause
                            assert.equal(queue.pause, 2);
                            return Promise.resolve([
                                {
                                    ReceiptHandle: "msg2",
                                    Body: '{"title":"plop2"}'
                                }
                            ]);
                        case 5:
                            // Error on callback dont generate a double delay
                            assert.equal(queue.pause, 2);
                            resolve(queue.stop());
                    }
                };
                queue.deleteMessage = handle => {
                    // Should only have the msg1 handle in deleteMessage as msg2 is fake error
                    assert.equal(handle, "msg1");
                };
                let callback = event => {
                    switch (seq) {
                        case 2:
                            assert.equal(event.title, "plop");
                            return;
                        case 4:
                            // Simulate error in callback
                            throw Error();
                            return;
                    }
                };
                queue.worker(callback);
            });
        });
    }
    basic() {
        return __awaiter(this, void 0, void 0, function* () {
            let queue = this.getService("memoryqueue");
            // For coverage
            assert.equal(queue._params.expire, 1000);
            queue._params.expire = undefined;
            yield queue.init();
            assert.equal(queue._params.expire, 30000);
            queue._params.expire = 1000;
            queue.__clean();
            return this.simple(queue);
        });
    }
};
__decorate([
    mocha_typescript_1.test
], MemoryQueueTest.prototype, "abstractQueue", null);
__decorate([
    mocha_typescript_1.test
], MemoryQueueTest.prototype, "worker", null);
__decorate([
    mocha_typescript_1.test
], MemoryQueueTest.prototype, "basic", null);
MemoryQueueTest = __decorate([
    mocha_typescript_1.suite
], MemoryQueueTest);
//# sourceMappingURL=memoryqueue.spec.js.map