import { Executor, Ident, Store, Context, Mailer, Service, User, Core } from "../index";
interface PasswordVerifier extends Service {
    validate(password: string): Promise<void>;
}
declare class PasswordRecoveryInfos {
    expire: number;
    token: string;
    login: string;
}
/**
 * This class is known as the Authentication module
 * It handles OAuth for several providers for now (Facebook, Google, Amazon, GitHub and Twitter)
 * It also handles email authentication with prevalidation or postvalidation of the email
 *
 * It requires two Store to work one 'idents' and one 'users'
 *
 * The parameters are
 *
 *   providerName: {
 *     clientID: '...',
 *     clientSecret: '...',
 *     scope: ''
 *   },
 *   email: {
 *	    postValidation: true|false   // If postValidation=true, account created without email verification
 *   }
 *   expose: 'url' // By default /auth
 *
 */
declare class Authentication extends Executor {
    /** @ignore */
    _identsStore: Store<Ident>;
    _usersStore: Store<User>;
    _passwordVerifier: PasswordVerifier;
    _aliases: Map<string, string>;
    _emailDelay: number;
    _oauth1: any;
    constructor(webda: Core, name: string, params: any);
    initRoutes(): void;
    _googleToken(context: Context): Promise<void>;
    setIdents(identStore: any): void;
    setUsers(userStore: any): void;
    /**
     * @ignore
     * Setup the default routes
     */
    init(): Promise<void>;
    addProvider(name: string, strategy: any, config: any): void;
    _sendEmailValidation(ctx: any): Promise<void>;
    _callback(ctx: Context): Promise<unknown>;
    _getMe(ctx: Context): Promise<void>;
    _listAuthentications(ctx: Context): Promise<void>;
    getCallbackUrl(ctx: Context): any;
    _registerNewEmail(ctx: any): Promise<void>;
    handleOAuthReturn(ctx: Context, identArg: any, done: any, noRedirect?: boolean): Promise<void>;
    setupOAuth(ctx: Context, config: any, name: string): void;
    registerUser(ctx: Context, datas: any, user?: any): Promise<any>;
    getPasswordRecoveryInfos(uuid: string | User, interval?: number): Promise<PasswordRecoveryInfos>;
    _passwordRecoveryEmail(ctx: Context): Promise<void>;
    _verifyPassword(password: string): Promise<void> | Promise<boolean>;
    _passwordRecovery(ctx: Context): Promise<void>;
    _handleEmailCallback(ctx: Context): Promise<void>;
    sendRecoveryEmail(ctx: Context, user: any, email: string): Promise<any>;
    sendValidationEmail(ctx: Context, email: string): Promise<any>;
    hashPassword(pass: string): string;
    logout(ctx: Context): Promise<void>;
    login(ctx: Context, user: any, ident: any): Promise<any[]>;
    getMailMan(): Mailer;
    protected handleLogin(ctx: Context, ident: Ident): Promise<void>;
    _handleEmail(ctx: Context): Promise<any>;
    generateEmailValidationToken(user: string, email: string): string;
    _getProviderName(name: string): any;
    _authenticate(ctx: Context): Promise<any>;
    static getModda(): {
        uuid: string;
        label: string;
        description: string;
        webcomponents: any[];
        logo: string;
        documentation: string;
        configuration: {
            default: {
                successRedirect: string;
                failureRedirect: string;
                providers: {
                    facebook: {
                        clientID: string;
                        clientSecret: string;
                        scope: string[];
                    };
                    email: {
                        postValidation: boolean;
                    };
                };
            };
            schema: {
                type: string;
                properties: {
                    expose: {
                        type: string;
                    };
                    successRedirect: {
                        type: string;
                    };
                    failureRedirect: {
                        type: string;
                    };
                    providers: {
                        type: string;
                    };
                };
                required: string[];
            };
        };
    };
}
export { Authentication, PasswordVerifier, PasswordRecoveryInfos };
