"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_1 = require("../test");
const assert = require("assert");
const __1 = require("..");
const mocha_typescript_1 = require("mocha-typescript");
const validationUrl = /.*\/auth\/email\/callback\?email=([^&]+)&token=([^& ]+)(&user=([^ &]+))?/;
var userId;
let AuthenticationTest = class AuthenticationTest extends test_1.WebdaTest {
    constructor() {
        super(...arguments);
        this.events = 0;
    }
    getUserStore() {
        return this.getService("Users");
    }
    getIdentStore() {
        return this.getService("Idents");
    }
    before() {
        const _super = Object.create(null, {
            before: { get: () => super.before }
        });
        return __awaiter(this, void 0, void 0, function* () {
            yield _super.before.call(this);
            this.userStore = this.getUserStore();
            this.identStore = this.getIdentStore();
            assert.notEqual(this.userStore, undefined);
            assert.notEqual(this.identStore, undefined);
            yield this.userStore.__clean();
            yield this.identStore.__clean();
            this.authentication = this.getService("Authentication");
            this.mailer = this.getService("DefinedMailer");
            this.authentication.on("Login", () => {
                this.events++;
            });
            this.authentication.on("Register", evt => {
                this.events++;
                evt.user.test = "TESTOR";
            });
            this.mailer.sent = [];
        });
    }
    registerTest2(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            let executor = this.getExecutor(ctx, "test.webda.io", "POST", "/auth/email", {
                login: "Test2@Webda.io",
                password: "testtest",
                register: true
            }, {
                "Accept-Language": "en-GB,en;q=0.9,fr;q=0.8"
            });
            ctx.newSession();
            executor._params.providers.email.postValidation = true;
            yield executor.execute(ctx);
        });
    }
    register() {
        return __awaiter(this, void 0, void 0, function* () {
            let ctx = yield this.newContext();
            let executor = this.getExecutor(ctx, "test.webda.io", "POST", "/auth/email", {
                email: "test@Webda.io",
                password: "testtest"
            }, {
                "Accept-Language": "en-GB,en;q=0.9,fr;q=0.8"
            });
            this.events = 0;
            yield this.assertThrowsAsync(executor.execute.bind(executor, ctx), res => res == 400);
            executor = this.getExecutor(ctx, "test.webda.io", "POST", "/auth/email", {
                login: "test@Webda.io",
                password: "testtest"
            }, {
                "Accept-Language": "en-GB,en;q=0.9,fr;q=0.8"
            });
            // User unknown without register parameter
            yield this.assertThrowsAsync(executor.execute.bind(executor, ctx), res => res == 404);
            executor = this.getExecutor(ctx, "test.webda.io", "POST", "/auth/email", {
                login: "test@Webda.io",
                password: "testtest",
                register: true
            }, {
                "Accept-Language": "en-GB,en;q=0.9,fr;q=0.8"
            });
            ctx.newSession();
            yield executor.execute(ctx);
            assert.equal(ctx.getSession().identId, undefined);
            assert.equal(this.mailer.sent.length, 1);
            let ident = yield this.identStore.get("test@webda.io_email");
            assert.equal(ident, undefined);
            // Not registered as postValidation is not set
            executor._params.providers.email.postValidation = true;
            yield this.registerTest2(ctx);
            assert.equal(this.events, 2); // Register + Login
            userId = ctx.getSession().getUserId();
            assert.notEqual(ctx.getSession().getUserId(), undefined);
            assert.equal(this.mailer.sent.length, 2);
            let user = yield this.userStore.get(ctx.getSession().getUserId());
            assert.notEqual(user, undefined);
            assert.notEqual(user.getPassword(), undefined);
            assert.equal(user.locale, "en");
            assert.equal(user.test, "TESTOR"); // Verify that the listener on Register has done something
            // Now logout
            executor = this.getExecutor(ctx, "test.webda.io", "DELETE", "/auth");
            yield executor.execute(ctx);
            // Now validate first user
            executor._params.providers.email.postValidation = false;
            assert.equal(ctx.getSession().getUserId(), undefined);
            var match = this.mailer.sent[0].replacements.url.match(validationUrl);
            assert.notEqual(match, undefined);
            assert.equal(match[1], "test@webda.io");
            executor = this.getExecutor(ctx, "test.webda.io", "POST", "/auth/email", {
                token: match[2],
                password: "testtest",
                login: match[1],
                register: true,
                add: "plop"
            });
            yield executor.execute(ctx);
            // Should create it with the dat provided
            assert.notEqual(ctx.getSession().getUserId(), undefined);
            ident = yield this.identStore.get(ctx.getSession().getIdentUsed());
            // Email should be already validate
            assert.notEqual(ident._validation, undefined);
            assert.equal(this.mailer.sent.length, 2);
            // Resend email
            executor = this.getExecutor(ctx, "test.webda.io", "GET", "/auth/email/test2@webda.io/validate");
            ctx.getSession().login(userId, "fake");
            // Should be to soon to resend an email
            yield this.assertThrowsAsync(executor.execute.bind(executor, ctx), res => res == 429);
            ident = yield this.identStore.get("test2@webda.io_email");
            ident._lastValidationEmail = 10;
            yield ident.save();
            // Should be ok now to send an email
            yield executor.execute(ctx);
            assert.equal(this.mailer.sent.length, 3);
            // Validate email for test2 now
            match = this.mailer.sent[1].replacements.url.match(validationUrl);
            assert.notEqual(match, undefined);
            assert.equal(match[1], "test2@webda.io");
            ctx.newSession();
            executor = this.getExecutor(ctx, "test.webda.io", "GET", "/auth/email/callback?email=" + match[1] + "&token=" + match[2]);
            yield executor.execute(ctx);
            assert.equal(ctx.statusCode, 302);
            // Verify the skipEmailValidation parameter
            this.events = 0;
            ctx.newSession();
            executor = this.getExecutor(ctx, "test.webda.io", "POST", "/auth/email", {
                login: "test4@webda.io",
                password: "testtest",
                register: true
            });
            executor._params.providers.email.postValidation = true;
            executor._params.providers.email.skipEmailValidation = true;
            yield executor.execute(ctx);
            // No new email has been sent
            assert.equal(this.mailer.sent.length, 3);
            assert.equal(this.events, 2); // Register + Login
            assert.notEqual(ctx.getSession().getUserId(), undefined);
        });
    }
    registerBadPassword() {
        return __awaiter(this, void 0, void 0, function* () {
            // By default a password of 8 is needed
            var params = {
                login: "testBad@Webda.io",
                password: "test",
                register: true
            };
            let ctx = yield this.newContext(params);
            let executor = this.getExecutor(ctx, "test.webda.io", "POST", "/auth/email", {}, {
                "Accept-Language": "en-GB"
            });
            // Activate post validation
            executor._params.providers.email.postValidation = true;
            var error = false;
            yield this.assertThrowsAsync(executor.execute.bind(executor, ctx), res => res == 400);
        });
    }
    me() {
        return __awaiter(this, void 0, void 0, function* () {
            let ctx = yield this.newContext();
            let executor = this.getExecutor(ctx, "test.webda.io", "GET", "/auth/me");
            yield this.assertThrowsAsync(executor.execute.bind(executor, ctx), res => res == 404);
            executor = this.getExecutor(ctx, "test.webda.io", "POST", "/auth/email", {
                login: "test5@webda.io",
                password: "testtest",
                register: true,
                plop: "yep"
            });
            executor._params.providers.email.postValidation = true;
            yield executor.execute(ctx);
            // Get me on known user
            executor = this.getExecutor(ctx, "test.webda.io", "GET", "/auth/me");
            yield executor.execute(ctx);
            let user = JSON.parse(ctx.getResponseBody());
            assert.equal(user.plop, "yep");
            assert.equal(user.register, undefined);
            assert.equal(user.locale, "es-ES");
            assert.notEqual(user, undefined);
        });
    }
    login() {
        return __awaiter(this, void 0, void 0, function* () {
            let ctx = yield this.newContext();
            yield this.registerTest2(ctx);
            ctx.newSession();
            this.events = 0;
            let executor = this.getExecutor(ctx, "test.webda.io", "POST", "/auth/email", {
                login: "test3@webda.io",
                password: "testtest"
            });
            yield this.assertThrowsAsync(executor.execute.bind(executor, ctx), res => res == 404);
            assert.equal(ctx.getSession().getUserId(), undefined);
            // As it has not been validate
            ctx.getHttpContext().setBody({
                login: "test2@webda.io",
                password: "testtest"
            });
            ctx.reinit();
            ctx.newSession();
            yield executor.execute(ctx);
            assert.equal(this.events, 1); // Login
            assert.notEqual(ctx.getSession().getUserId(), undefined);
            // Verify ident type
            let ident = yield this.identStore.get(ctx.getSession().getIdentUsed());
            assert.equal(ident._type, "email");
            ctx.getHttpContext().setBody({
                login: "test2@webda.io",
                password: "bouzouf"
            });
            ctx.reinit();
            ctx.newSession();
            yield this.assertThrowsAsync(executor.execute.bind(executor, ctx), res => res == 403);
            assert.equal(ctx.getSession().getUserId(), undefined);
            assert.equal(this.events, 1);
        });
    }
    testPasswordRecovery() {
        return __awaiter(this, void 0, void 0, function* () {
            let tokenInfo;
            let ctx = yield this.newContext();
            yield this.registerTest2(ctx);
            this.mailer.sent = [];
            this.events = 0;
            userId = ctx.getSession().getUserId();
            ctx.newSession();
            let executor = this.getExecutor(ctx, "test.webda.io", "POST", "/auth/email/passwordRecovery", {
                login: userId,
                password: "retesttest"
            });
            tokenInfo = yield this.authentication.getPasswordRecoveryInfos(userId, -10);
            yield this.assertThrowsAsync(executor.execute.bind(executor, ctx), res => res == 400);
            // Missing the body
            executor = this.getExecutor(ctx, "test.webda.io", "POST", "/auth/email/passwordRecovery", {
                token: tokenInfo.token,
                expire: 123,
                password: "retesttest",
                login: userId
            });
            yield this.assertThrowsAsync(executor.execute.bind(executor, ctx), res => res == 403);
            // Missing the body
            ctx.getHttpContext().setBody({
                token: tokenInfo.token,
                expire: tokenInfo.expire,
                password: "retesttest",
                login: userId
            });
            ctx.reinit();
            yield this.assertThrowsAsync(executor.execute.bind(executor, ctx), res => res == 410);
            tokenInfo = yield this.authentication.getPasswordRecoveryInfos(userId);
            // Missing the body
            ctx
                .getHttpContext()
                .setBody(Object.assign(Object.assign({}, tokenInfo), { login: userId, password: "retesttest" }));
            ctx.reinit();
            yield executor.execute(ctx);
            // Should be update with password retest now
            ctx.newSession();
            executor = this.getExecutor(ctx, "test.webda.io", "POST", "/auth/email", {
                login: "test2@webda.io",
                password: "retesttest"
            });
            yield executor.execute(ctx);
            assert.notEqual(ctx.getSession().getUserId(), undefined);
            executor = this.getExecutor(ctx, "test.webda.io", "GET", "/auth/email/test2@webda.io/recover");
            yield executor.execute(ctx);
            assert.equal(this.mailer.sent.length, 1);
            assert.equal(this.mailer.sent[0].to, "test2@webda.io");
            assert.notEqual(this.mailer.sent[0].replacements.infos, undefined);
            assert.notEqual(this.mailer.sent[0].replacements.infos.expire, undefined);
            assert.notEqual(this.mailer.sent[0].replacements.infos.token, undefined);
            yield this.assertThrowsAsync(executor.execute.bind(executor, ctx), res => res == 429);
        });
    }
    addEmailToAccount() {
        return __awaiter(this, void 0, void 0, function* () {
            // Log in as test2
            let ctx = yield this.newContext();
            yield this.registerTest2(ctx);
            this.mailer.sent = [];
            let executor = this.getExecutor(ctx, "test.webda.io", "POST", "/auth/email", {
                login: "test2@webda.io",
                password: "testtest"
            });
            yield executor.execute(ctx);
            assert.notEqual(ctx.getSession().getUserId(), undefined);
            executor = this.getExecutor(ctx, "test.webda.io", "GET", "/auth/email/newtest@webda.io/validate", {
                login: "test2@webda.io",
                password: "retesttest"
            });
            yield executor.execute(ctx);
            let ident = yield this.identStore.get("newtest@webda.io_email");
            assert.notEqual(ident, undefined);
            assert.equal(ident._validation, undefined);
            userId = ctx.getCurrentUserId();
            var match = this.mailer.sent[0].replacements.url.match(validationUrl);
            assert.notEqual(match, undefined);
            assert.equal(match[1], "newtest@webda.io");
            // Send another one on newtest2
            executor = this.getExecutor(ctx, "test.webda.io", "GET", "/auth/email/newtest2@webda.io/validate", {
                login: "test2@webda.io",
                password: "retesttest"
            });
            yield executor.execute(ctx);
            var match2 = this.mailer.sent[1].replacements.url.match(validationUrl);
            assert.notEqual(match2, undefined);
            assert.equal(match2[1], "newtest2@webda.io");
            // Try to validate with wrong email
            ctx.getSession().userId = "anotheruser";
            executor = this.getExecutor(ctx, "test.webda.io", "GET", "/auth/email/callback?email=" +
                match[1] +
                "&token=" +
                match[2] +
                "&user=" +
                match[4]);
            yield executor.execute(ctx);
            assert.equal(ctx.statusCode, 302);
            assert.equal(ctx.getResponseHeaders().Location, "/login-error?reason=badUser");
            // Right user
            ctx.getSession().userId = userId;
            executor = this.getExecutor(ctx, "test.webda.io", "GET", "/auth/email/callback?email=" +
                match[1] +
                "&token=bouzouf" +
                "&user=" +
                match[4]);
            yield executor.execute(ctx);
            assert.equal(ctx.statusCode, 302);
            assert.equal(ctx.getResponseHeaders().Location, "/login-error?reason=badToken");
            executor = this.getExecutor(ctx, "test.webda.io", "GET", "/auth/email/callback?email=" +
                match[1] +
                "&token=" +
                match[2] +
                "&user=" +
                match[4]);
            yield executor.execute(ctx);
            assert.equal(ctx.statusCode, 302);
            assert.equal(ctx.getResponseHeaders().Location, "https://webda.io/user.html?validation=email");
            ident = yield this.identStore.get("newtest@webda.io_email");
            assert.notEqual(ident._validation, undefined);
            // Verify exception if same user try to revalidate
            executor = this.getExecutor(ctx, "test.webda.io", "GET", "/auth/email/newtest@webda.io/validate");
            yield this.assertThrowsAsync(executor.execute.bind(executor, ctx), res => res == 412);
            // Check ident is added to the user
            // TODO Redo this one
            /*
            let user = await ctx.getCurrentUser();
            assert.equal(
              user.getIdents().filter(item => item.uuid === "newtest@webda.io_email")
                .length,
              1
            );
            */
            // Verify exception if different user try to validate
            ctx.getSession().userId = "bouzouf";
            executor = this.getExecutor(ctx, "test.webda.io", "GET", "/auth/email/newtest@webda.io/validate");
            yield this.assertThrowsAsync(executor.execute.bind(executor, ctx), res => res == 409);
            // Validation with no user
            ctx.newSession();
            executor = this.getExecutor(ctx, "test.webda.io", "GET", "/auth/email/callback?email=" +
                match2[1] +
                "&token=" +
                match2[2] +
                "&user=" +
                match2[4]);
            yield executor.execute(ctx);
            assert.equal(ctx.statusCode, 302);
            assert.equal(ctx.getResponseHeaders().Location, "https://webda.io/user.html?validation=email");
        });
    }
    awsCompatibility() {
        return __awaiter(this, void 0, void 0, function* () {
            let ctx = yield this.newContext();
            let executor = this.getExecutor(ctx, "test.webda.io", "GET", "/auth/github");
            assert.notEqual(executor, undefined);
        });
    }
    callback() {
        return __awaiter(this, void 0, void 0, function* () {
            var done = function () { };
            var lastUsed = null;
            this.events = 0;
            let ident = __1.Ident.init("github", "test");
            let ctx = yield this.newContext();
            this.getExecutor(ctx, "test.webda.io", "GET", "/auth/github/callback?code=blahblah");
            yield this.authentication.handleOAuthReturn(ctx, ident, done);
            assert.equal(ctx.statusCode, 302);
            assert.equal(ctx.parameter("code"), "blahblah");
            assert.equal(ctx.getResponseHeaders().Location, "https://webda.io/user.html?validation=github");
            ident = yield this.identStore.get(ident.uuid);
            // The ident must have been created and have a last used
            assert.notEqual(ident._lastUsed, lastUsed);
            lastUsed = ident._lastUsed;
            // Set by the store
            assert.notEqual(ident._lastUpdate, undefined);
            // Login + Register
            assert.equal(this.events, 2);
            this.events = 0;
            assert.equal(ctx.getSession().isLogged(), true);
            yield this.sleep(50);
            yield this.authentication.handleOAuthReturn(ctx, ident, done);
            yield ident.refresh();
            assert.equal(this.events, 1); // Only Login
            assert.notEqual(ident._lastUsed, lastUsed);
            let user = yield this.userStore.get(ident._user);
            this.events = 0;
            assert.equal(user.idents.length, 1); // Only one github login
            assert.equal(user.idents[0].uuid, "test_github"); // Only one github login
            assert.equal(user.test, "TESTOR"); // Verify that the listener on Register has done something
            yield this.authentication.handleOAuthReturn(ctx, __1.Ident.init("github", "retest"), done);
            assert.equal(this.events, 1); // Only Login
            assert.equal(ctx.statusCode, 302);
            assert.equal(ctx.getResponseHeaders().Location, "https://webda.io/user.html?validation=github");
            user = yield this.userStore.get(ident._user);
            assert.equal(user.idents.length, 2); // Two github login
            assert.equal(user.idents[1].uuid, "retest_github");
        });
    }
};
__decorate([
    mocha_typescript_1.test("register")
], AuthenticationTest.prototype, "register", null);
__decorate([
    mocha_typescript_1.test("register - bad password")
], AuthenticationTest.prototype, "registerBadPassword", null);
__decorate([
    mocha_typescript_1.test("/me")
], AuthenticationTest.prototype, "me", null);
__decorate([
    mocha_typescript_1.test("login")
], AuthenticationTest.prototype, "login", null);
__decorate([
    mocha_typescript_1.test("passwordRecovery")
], AuthenticationTest.prototype, "testPasswordRecovery", null);
__decorate([
    mocha_typescript_1.test("add email to existing account")
], AuthenticationTest.prototype, "addEmailToAccount", null);
__decorate([
    mocha_typescript_1.test("AWS Compatibility")
], AuthenticationTest.prototype, "awsCompatibility", null);
__decorate([
    mocha_typescript_1.test("Callback")
], AuthenticationTest.prototype, "callback", null);
AuthenticationTest = __decorate([
    mocha_typescript_1.suite
], AuthenticationTest);
//# sourceMappingURL=authentication.spec.js.map