"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_1 = require("../test");
const assert = require("assert");
const fs = require("fs");
const mocha_typescript_1 = require("mocha-typescript");
class BinaryTest extends test_1.WebdaTest {
    getUserStore() {
        return this.getService("Users");
    }
    getBinary() {
        return this.getService("binary");
    }
    getMap() {
        return "images";
    }
    getExposePath() {
        return "/binary";
    }
    getTestFile() {
        return process.cwd() + "/test/Dockerfile.txt";
    }
    before() {
        const _super = Object.create(null, {
            before: { get: () => super.before }
        });
        return __awaiter(this, void 0, void 0, function* () {
            yield _super.before.call(this);
            let userStore = this.getUserStore();
            let binary = this.getBinary();
            assert.notEqual(userStore, undefined);
            assert.notEqual(binary, undefined);
            yield userStore.__clean();
            yield binary.__clean();
        });
    }
    normal() {
        return __awaiter(this, void 0, void 0, function* () {
            let userStore = this.getUserStore();
            let binary = this.getBinary();
            let map = this.getMap();
            let exposePath = this.getExposePath();
            var eventFired = 0;
            var events = ["binaryGet", "binaryUpdate", "binaryCreate", "binaryDelete"];
            for (let evt in events) {
                binary.on(events[evt], function () {
                    eventFired++;
                });
            }
            // Check CREATE - READ
            var hash;
            var user1;
            var user2;
            var user;
            var ctx;
            var error;
            user1 = yield userStore.save({
                test: "plop"
            });
            user2 = yield userStore.save({
                test: "plop"
            });
            yield binary.store(userStore, user1, map, {
                path: this.getTestFile()
            }, {});
            user1 = yield userStore.get(user1.uuid);
            assert.notEqual(user1[map], undefined);
            assert.equal(user1[map].length, 1);
            hash = user1[map][0].hash;
            let value = yield binary.getUsageCount(hash);
            assert.equal(value, 1);
            yield binary.store(userStore, user2, map, {
                path: this.getTestFile()
            }, {});
            user = yield userStore.get(user2.uuid);
            assert.notEqual(user[map], undefined);
            assert.equal(user[map].length, 1);
            assert.equal(user[map][0].constructor.name, "BinaryMap");
            hash = user[map][0].hash;
            value = yield binary.getUsageCount(hash);
            assert.equal(value, 2);
            yield binary.delete(userStore, user, map, 0);
            user = yield userStore.get(user2.uuid);
            assert.equal(user[map].length, 0);
            value = yield binary.getUsageCount(hash);
            assert.equal(value, 1);
            // Try to get images on user1 as user2
            ctx = yield this.newContext({
                type: "CRUD",
                uuid: "PLOP"
            });
            ctx.session.userId = user2.uuid;
            let executor = this.getExecutor(ctx, "test.webda.io", "GET", exposePath + "/users/" + user1.uuid + "/" + map + "/0");
            yield this.assertThrowsAsync(executor.execute.bind(executor, ctx), res => res == 403);
            ctx.session.userId = user1.uuid;
            executor = this.getExecutor(ctx, "test.webda.io", "GET", exposePath + "/users/" + user1.uuid + "/" + map + "/0");
            yield executor.execute(ctx);
            // We dont check for result as FileBinary will return datas and S3 a redirect
            if (fs.existsSync("./downloadTo.tmp")) {
                fs.unlinkSync("./downloadTo.tmp");
            }
            yield binary.downloadTo(user1[map][0], "./downloadTo.tmp");
            // Check the result is the same
            assert.equal(fs.readFileSync("./downloadTo.tmp").toString(), fs.readFileSync(this.getTestFile()).toString());
            yield userStore.delete(user1.uuid);
            value = yield binary.getUsageCount(hash);
            assert.equal(value, 0);
        });
    }
    notMapped() {
        return __awaiter(this, void 0, void 0, function* () {
            let userStore = this.getUserStore();
            let binary = this.getBinary();
            let user1 = yield userStore.save({
                test: "plop"
            });
            yield this.assertThrowsAsync(binary.store.bind(binary, userStore, user1, "images2", {
                path: this.getTestFile()
            }, {}), err => true);
        });
    }
    update() {
        return __awaiter(this, void 0, void 0, function* () {
            let userStore = this.getUserStore();
            let binary = this.getBinary();
            let map = this.getMap();
            var user1;
            var user;
            let hash;
            user1 = yield userStore.save({
                test: "plop"
            });
            yield binary.store(userStore, user1, map, {
                path: this.getTestFile()
            }, {});
            user = yield userStore.get(user1.uuid);
            assert.notEqual(user[map], undefined);
            assert.equal(user[map].length, 1);
            let value = yield binary.getUsageCount(user[map][0].hash);
            assert.equal(value, 1);
            yield binary.update(userStore, user, map, 0, {
                path: this.getTestFile()
            }, {});
            value = yield binary.getUsageCount(user[map][0].hash);
            assert.equal(value, 1);
            user = yield userStore.get(user1.uuid);
            assert.notEqual(user[map], undefined);
            assert.equal(user[map].length, 1);
            assert.notEqual(hash, user[map][0].hash);
            assert.equal(user[map][0].mimetype, "text/plain");
            assert.equal(user[map][0].name, "Dockerfile.txt");
            value = yield binary.getUsageCount(hash);
            assert.equal(value, 0);
            value = yield binary.getUsageCount(user[map][0].hash);
            assert.equal(value, 1);
        });
    }
}
__decorate([
    mocha_typescript_1.test
], BinaryTest.prototype, "normal", null);
__decorate([
    mocha_typescript_1.test
], BinaryTest.prototype, "notMapped", null);
__decorate([
    mocha_typescript_1.test
], BinaryTest.prototype, "update", null);
exports.BinaryTest = BinaryTest;
//# sourceMappingURL=binary.spec.js.map