"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("../index");
/**
 * Handle sessionSecret ( rolling between two secrets ) expire every hour
 * Handle longTermSecret ( rolling between two longer secret ) expire every month
 */
class ConfigurationService extends index_1.Service {
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            // Check interval by default every hour
            if (!this._params.checkInterval) {
                this._params.checkInterval = 3600;
            }
            if (!this._params.source) {
                throw new Error("Need a source for ConfigurationService");
            }
            let source = this._params.source.split(":");
            this._sourceService = this.getService(source[0]);
            if (!this._sourceService) {
                throw new Error('Need a valid service for source ("sourceService:sourceId")');
            }
            this._sourceId = source[1];
            if (!this._sourceId) {
                throw new Error('Need a valid source ("sourceService:sourceId")');
            }
            if (!this._sourceService.getConfiguration) {
                throw new Error(`Service ${source[0]} is not implementing ConfigurationProvider interface`);
            }
            this._configuration = JSON.stringify(this._params.default);
            yield this._checkUpdate();
            this._interval = setInterval(this._checkUpdate.bind(this), 1000);
        });
    }
    stop() {
        // @ts-ignore
        clearInterval(this._interval);
    }
    reinit(config) {
        return __awaiter(this, void 0, void 0, function* () {
            // Need to prevent any reinit
        });
    }
    _loadConfiguration() {
        return __awaiter(this, void 0, void 0, function* () {
            return this._sourceService.getConfiguration(this._sourceId);
        });
    }
    _checkUpdate() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._nextCheck > new Date().getTime())
                return;
            this.log("DEBUG", "Refreshing configuration");
            let newConfig = (yield this._loadConfiguration()) || this._params.default;
            if (JSON.stringify(newConfig) !== this._configuration) {
                this.log("DEBUG", "Apply new configuration");
                this._configuration = JSON.stringify(newConfig);
                this._webda.reinit(newConfig);
            }
            this._updateNextCheck();
            this.log("DEBUG", "Next configuration refresh in", this._params.checkInterval, "s");
        });
    }
    _updateNextCheck() {
        this._nextCheck = new Date().getTime() + this._params.checkInterval * 1000;
    }
}
exports.default = ConfigurationService;
exports.ConfigurationService = ConfigurationService;
//# sourceMappingURL=configuration.js.map