"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("../index");
const fs = require("fs");
/**
 * Simple file storage of object
 *
 * Storage structure
 *   /folder/{uuid}
 *
 *
 * Parameters:
 *  folder: to store to
 *
 */
class FileStore extends index_1.Store {
    /** @ignore */
    constructor(webda, name, options) {
        super(webda, name, options);
        if (!fs.existsSync(options.folder)) {
            fs.mkdirSync(options.folder);
        }
    }
    file(uid) {
        return this._params.folder + "/" + uid;
    }
    exists(uid) {
        return __awaiter(this, void 0, void 0, function* () {
            // existsSync is deprecated might change it
            return Promise.resolve(fs.existsSync(this.file(uid)));
        });
    }
    _find(request, offset, limit) {
        return __awaiter(this, void 0, void 0, function* () {
            var self = this;
            var res = [];
            var path = require("path");
            var files = fs.readdirSync(self._params.folder).filter(function (file) {
                return !fs.statSync(path.join(self._params.folder, file)).isDirectory();
            });
            for (var file in files) {
                res.push(this._get(files[file]));
            }
            return Promise.all(res);
        });
    }
    _save(object, uid) {
        fs.writeFileSync(this.file(uid), JSON.stringify(object.toStoredJSON(), undefined, this._params.beautify));
        return Promise.resolve(object);
    }
    _upsertItemToCollection(uid, prop, item, index, itemWriteCondition, itemWriteConditionField, updateDate) {
        return __awaiter(this, void 0, void 0, function* () {
            let res = yield this._get(uid);
            if (res === undefined) {
                throw Error("NotFound");
            }
            if (index === undefined) {
                if (itemWriteCondition !== undefined &&
                    res[prop].length !== itemWriteCondition) {
                    throw Error("UpdateCondition not met");
                }
                if (res[prop] === undefined) {
                    res[prop] = [item];
                }
                else {
                    res[prop].push(item);
                }
            }
            else {
                if (itemWriteCondition &&
                    res[prop][index][itemWriteConditionField] != itemWriteCondition) {
                    throw Error("UpdateCondition not met");
                }
                res[prop][index] = item;
            }
            res[this._lastUpdateField] = updateDate;
            yield this._save(res, uid);
        });
    }
    _removeAttribute(uuid, attribute) {
        return __awaiter(this, void 0, void 0, function* () {
            let res = yield this._get(uuid);
            delete res[attribute];
            yield this._save(res, uuid);
        });
    }
    _deleteItemFromCollection(uid, prop, index, itemWriteCondition, itemWriteConditionField, updateDate) {
        return __awaiter(this, void 0, void 0, function* () {
            let res = yield this._get(uid);
            if (res === undefined) {
                throw Error("NotFound");
            }
            if (itemWriteCondition &&
                res[prop][index][itemWriteConditionField] != itemWriteCondition) {
                throw Error("UpdateCondition not met");
            }
            res[prop].splice(index, 1);
            res[this._lastUpdateField] = updateDate;
            return this._save(res, uid);
        });
    }
    _delete(uid, writeCondition) {
        return __awaiter(this, void 0, void 0, function* () {
            let res = yield this._get(uid);
            if (writeCondition &&
                res &&
                res[this._writeConditionField] != writeCondition) {
                return Promise.reject(Error("UpdateCondition not met"));
            }
            if (res) {
                fs.unlinkSync(this.file(uid));
            }
            return Promise.resolve();
        });
    }
    _patch(object, uid, writeCondition) {
        return __awaiter(this, void 0, void 0, function* () {
            let stored = yield this._get(uid);
            if (!stored) {
                return Promise.reject(Error("NotFound"));
            }
            if (writeCondition && stored[this._writeConditionField] != writeCondition) {
                return Promise.reject(Error("UpdateCondition not met"));
            }
            for (var prop in object) {
                stored[prop] = object[prop];
            }
            return this._save(stored, uid);
        });
    }
    _update(object, uid, writeCondition = undefined) {
        return __awaiter(this, void 0, void 0, function* () {
            let stored = yield this._get(uid);
            if (!stored) {
                throw new Error("NotFound");
            }
            if (writeCondition && stored[this._writeConditionField] != writeCondition) {
                throw new Error("UpdateCondition not met");
            }
            let coreModel = new index_1.CoreModel();
            coreModel.load(object, true);
            return this._save(coreModel, uid);
        });
    }
    getAll(uids) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!uids) {
                uids = [];
                var files = fs.readdirSync(this._params.folder);
                for (var file in files) {
                    uids.push(files[file]);
                }
            }
            let result = [];
            for (let i in uids) {
                result.push(this._get(uids[i]));
            }
            return Promise.all(result);
        });
    }
    _get(uid) {
        return __awaiter(this, void 0, void 0, function* () {
            let res = yield this.exists(uid);
            if (res) {
                let data = fs.readFileSync(this.file(uid));
                return this.initModel(JSON.parse(data.toString()));
            }
            return;
        });
    }
    _incrementAttribute(uid, prop, value, updateDate) {
        return __awaiter(this, void 0, void 0, function* () {
            let found = this.exists(uid);
            if (!found) {
                throw Error("NotFound");
            }
            let stored = yield this._get(uid);
            if (stored[prop] === undefined) {
                stored[prop] = 0;
            }
            stored[this._lastUpdateField] = updateDate;
            stored[prop] += value;
            return this._save(stored, uid);
        });
    }
    __clean() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!fs.existsSync(this._params.folder)) {
                return;
            }
            var files = fs.readdirSync(this._params.folder);
            var promises = [];
            for (var file in files) {
                let filename = this._params.folder + "/" + files[file];
                promises.push(new Promise((resolve, reject) => {
                    fs.unlink(filename, err => {
                        if (err) {
                            reject(err);
                        }
                        resolve();
                    });
                }));
            }
            yield Promise.all(promises);
            if (this._params.index) {
                yield this.save({}, "index");
            }
        });
    }
    static getModda() {
        return {
            uuid: "Webda/FileStore",
            label: "File Store",
            description: "Implements user registration and login using either email or OAuth, it handles for now Facebook, Google, Amazon, GitHub, Twitter\nIt needs a Idents and a Users Store to work",
            webcomponents: [],
            documentation: "https://raw.githubusercontent.com/loopingz/webda/master/readmes/Store.md",
            logo: "images/icons/filedb.png",
            configuration: {
                default: {
                    folder: "/tmp/types"
                },
                widget: {
                    tag: "webda-store-configurator",
                    url: "elements/services/webda-store-configurator.html"
                },
                schema: {
                    type: "object",
                    properties: {
                        folder: {
                            type: "string"
                        }
                    },
                    required: ["folder"]
                }
            }
        };
    }
}
exports.FileStore = FileStore;
//# sourceMappingURL=file.js.map