"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const store_spec_1 = require("./store.spec");
const assert = require("assert");
const mocha_typescript_1 = require("mocha-typescript");
let FileStoreTest = class FileStoreTest extends store_spec_1.StoreTest {
    getUserStore() {
        return this.getService("Users");
    }
    getIdentStore() {
        return this.getService("Idents");
    }
    modelActions() {
        return __awaiter(this, void 0, void 0, function* () {
            let identStore = (this.getService("idents"));
            let eventFired = 0;
            let executor, ctx;
            identStore.on("Store.Action", evt => {
                eventFired++;
            });
            identStore.on("Store.Actioned", evt => {
                eventFired++;
            });
            ctx = yield this.newContext({
                type: "CRUD",
                uuid: "PLOP"
            });
            executor = this.getExecutor(ctx, "test.webda.io", "PUT", "/idents/coucou/plop");
            assert.notEqual(executor, undefined);
            yield this.assertThrowsAsync(executor.execute.bind(executor, ctx), err => err == 404);
            yield identStore.save({
                uuid: "coucou"
            });
            yield executor.execute(ctx);
            // Our fake action is pushing true to _plop
            assert.equal(JSON.parse(ctx.getResponseBody())._plop, true);
            assert.equal(eventFired, 2);
            assert.notEqual(this.getExecutor(ctx, "test.webda.io", "POST", "/idents/coucou/yop"), null);
            assert.notEqual(this.getExecutor(ctx, "test.webda.io", "GET", "/idents/coucou/yop"), null);
        });
    }
    modelStaticActions() {
        return __awaiter(this, void 0, void 0, function* () {
            let identStore = (this.getService("idents"));
            let ctx, executor;
            let eventFired = 0;
            identStore.on("Store.Action", evt => {
                eventFired++;
            });
            ctx = yield this.newContext({
                type: "CRUD",
                uuid: "PLOP"
            });
            executor = this.getExecutor(ctx, "test.webda.io", "GET", "/idents/index");
            assert.notEqual(executor, undefined);
            yield executor.execute(ctx);
            // Our fake index action is just outputing 'indexer'
            assert.equal(ctx.getResponseBody(), "indexer");
            assert.equal(eventFired, 1);
        });
    }
    httpCRUD() {
        return __awaiter(this, void 0, void 0, function* () {
            let eventFired;
            let userStore = this.getUserStore();
            let ctx, executor;
            yield userStore.__clean();
            ctx = yield this.newContext({});
            ctx.session.login("fake_user", "fake_ident");
            executor = this.getExecutor(ctx, "test.webda.io", "POST", "/users", {
                type: "CRUD",
                uuid: "PLOP"
            });
            assert.notEqual(executor, undefined);
            yield executor.execute(ctx);
            ctx.body = undefined;
            yield this.getExecutor(ctx, "test.webda.io", "GET", "/users/PLOP").execute(ctx);
            assert.notEqual(ctx.getResponseBody(), undefined);
            assert.equal(ctx.getResponseBody().indexOf("_lastUpdate") >= 0, true);
            executor = this.getExecutor(ctx, "test.webda.io", "POST", "/users", {
                type: "CRUD2",
                uuid: "PLOP"
            });
            yield this.assertThrowsAsync(executor.execute.bind(executor, ctx), err => err == 409);
            // Verify the none overide of UUID
            executor = this.getExecutor(ctx, "test.webda.io", "PUT", "/users/PLOP", {
                type: "CRUD2",
                additional: "field",
                uuid: "PLOP2",
                user: "fake_user"
            });
            yield executor.execute(ctx);
            let user = yield userStore.get("PLOP");
            assert.equal(user.uuid, "PLOP");
            assert.equal(user.type, "CRUD2");
            assert.equal(user.additional, "field");
            assert.equal(user.user, "fake_user");
            ctx.resetResponse();
            // Check PATH
            executor = this.getExecutor(ctx, "test.webda.io", "PATCH", "/users/PLOP", {
                type: "CRUD3",
                uuid: "PLOP2",
                _testor: "_ should not be update by client"
            });
            yield executor.execute(ctx);
            user = yield userStore.get("PLOP");
            assert.equal(user.uuid, "PLOP");
            assert.equal(user.type, "CRUD3");
            assert.equal(user.additional, "field");
            assert.equal(user._testor, undefined);
            executor = this.getExecutor(ctx, "test.webda.io", "PUT", "/users/PLOP", {
                type: "CRUD3",
                uuid: "PLOP2",
                _testor: "_ should not be update by client"
            });
            yield executor.execute(ctx);
            user = yield userStore.get("PLOP");
            assert.equal(user.uuid, "PLOP");
            assert.equal(user.type, "CRUD3");
            assert.equal(user.additional, undefined);
            assert.equal(user._testor, undefined);
            yield this.getExecutor(ctx, "test.webda.io", "DELETE", "/users/PLOP").execute(ctx);
            eventFired = 0;
            executor = this.getExecutor(ctx, "test.webda.io", "GET", "/users/PLOP");
            yield this.assertThrowsAsync(executor.execute.bind(executor, ctx), err => err == 404);
            eventFired++;
            executor = this.getExecutor(ctx, "test.webda.io", "DELETE", "/users/PLOP");
            yield this.assertThrowsAsync(executor.execute.bind(executor, ctx), err => err == 404);
            eventFired++;
            executor = this.getExecutor(ctx, "test.webda.io", "PUT", "/users/PLOP");
            yield this.assertThrowsAsync(executor.execute.bind(executor, ctx), err => err == 404);
            eventFired++;
            assert.equal(eventFired, 3);
        });
    }
    getURL() {
        return __awaiter(this, void 0, void 0, function* () {
            assert.equal(this.webda.getService("users").getUrl(), "/users");
        });
    }
    schemaCreate() {
        return __awaiter(this, void 0, void 0, function* () {
            let taskStore = this.webda.getTypedService("Tasks");
            let ctx = yield this.webda.newContext(undefined);
            let executor = this.getExecutor(ctx, "test.webda.io", "POST", "/tasks", {
                noname: "Task #1"
            });
            assert.notEqual(executor, undefined);
            ctx.getSession().login("fake_user", "fake_ident");
            yield this.assertThrowsAsync(executor.execute.bind(executor, ctx), err => err == 400);
            executor = this.getExecutor(ctx, "test.webda.io", "POST", "/tasks", {
                name: "Task #1"
            });
            yield executor.execute(ctx);
            let task = JSON.parse(ctx.getResponseBody());
            // It is two because the Saved has been called two
            assert.notEqual(task.uuid, undefined);
            assert.equal(task._autoListener, 2);
            task = yield taskStore.get(task.uuid);
            assert.equal(task._autoListener, 1);
        });
    }
};
__decorate([
    mocha_typescript_1.test
], FileStoreTest.prototype, "modelActions", null);
__decorate([
    mocha_typescript_1.test
], FileStoreTest.prototype, "modelStaticActions", null);
__decorate([
    mocha_typescript_1.test
], FileStoreTest.prototype, "httpCRUD", null);
__decorate([
    mocha_typescript_1.test
], FileStoreTest.prototype, "getURL", null);
__decorate([
    mocha_typescript_1.test("JSON Schema - Create")
], FileStoreTest.prototype, "schemaCreate", null);
FileStoreTest = __decorate([
    mocha_typescript_1.suite
], FileStoreTest);
exports.FileStoreTest = FileStoreTest;
//# sourceMappingURL=file.spec.js.map