"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("../index");
class MemoryStore extends index_1.Store {
    constructor() {
        super(...arguments);
        this.storage = {};
    }
    init() {
        const _super = Object.create(null, {
            init: { get: () => super.init }
        });
        return __awaiter(this, void 0, void 0, function* () {
            return _super.init.call(this);
        });
    }
    exists(uid) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.storage[uid] !== undefined;
        });
    }
    _find(request, offset, limit) {
        return __awaiter(this, void 0, void 0, function* () {
            // Need to transfert to Array
            return this.storage;
        });
    }
    _save(object, uid) {
        return __awaiter(this, void 0, void 0, function* () {
            uid = uid || object.uuid;
            if (!(object instanceof this._model)) {
                object = this.initModel(object);
            }
            this.storage[uid] = object.toStoredJSON(true);
            return this._getSync(uid);
        });
    }
    _delete(uid) {
        return __awaiter(this, void 0, void 0, function* () {
            delete this.storage[uid];
        });
    }
    _patch(object, uid) {
        return __awaiter(this, void 0, void 0, function* () {
            uid = uid || object.uuid;
            let obj = this._getSync(uid);
            for (let prop in object) {
                obj[prop] = object[prop];
            }
            this.storage[uid] = obj.toStoredJSON(true);
            return this._getSync(uid);
        });
    }
    _update(object, uid) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._save(object, uid);
            return this._getSync(uid);
        });
    }
    getAll(uids) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!uids) {
                return Object.keys(this.storage).map(key => {
                    return this._getSync(key);
                });
            }
            let result = [];
            for (let i in uids) {
                if (this.storage[uids[i]]) {
                    result.push(this._getSync(uids[i]));
                }
            }
            return result;
        });
    }
    _getSync(uid) {
        if (this.storage[uid]) {
            return this.initModel(JSON.parse(this.storage[uid]));
        }
        return null;
    }
    _removeAttribute(uuid, attribute) {
        return __awaiter(this, void 0, void 0, function* () {
            let res = yield this._get(uuid);
            delete res[attribute];
            this._save(res, uuid);
        });
    }
    _get(uid) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.storage[uid])
                return;
            return this._getSync(uid);
        });
    }
    __clean() {
        return __awaiter(this, void 0, void 0, function* () {
            this.storage = {};
            if (this._params.index) {
                yield this.save({}, "index");
            }
        });
    }
    _incrementAttribute(uid, prop, value, updateDate) {
        return __awaiter(this, void 0, void 0, function* () {
            var res = this.storage[uid];
            if (res === undefined) {
                throw Error("NotFound");
            }
            res = this._getSync(uid);
            if (!res[prop]) {
                res[prop] = 0;
            }
            res[this._lastUpdateField] = updateDate;
            res[prop] += value;
            return this._save(res, uid);
        });
    }
    _upsertItemToCollection(uid, prop, item, index, itemWriteCondition, itemWriteConditionField, updateDate) {
        return __awaiter(this, void 0, void 0, function* () {
            var res = this.storage[uid];
            if (res === undefined) {
                throw Error("NotFound");
            }
            res = this._getSync(uid);
            if (index === undefined) {
                if (itemWriteCondition !== undefined &&
                    res[prop].length !== itemWriteCondition) {
                    console.log("met", itemWriteCondition, res[prop].length, itemWriteCondition);
                    throw Error("UpdateCondition not met");
                }
                if (res[prop] === undefined) {
                    res[prop] = [item];
                }
                else {
                    res[prop].push(item);
                }
            }
            else {
                if (itemWriteCondition &&
                    res[prop][index][itemWriteConditionField] != itemWriteCondition) {
                    throw Error("UpdateCondition not met");
                }
                res[prop][index] = item;
            }
            res[this._lastUpdateField] = updateDate;
            yield this._save(res, uid);
        });
    }
    _deleteItemFromCollection(uid, prop, index, itemWriteCondition, itemWriteConditionField, updateDate) {
        return __awaiter(this, void 0, void 0, function* () {
            var res = this.storage[uid];
            if (res === undefined) {
                throw Error("NotFound");
            }
            res = this._getSync(uid);
            if (itemWriteCondition &&
                res[prop][index][itemWriteConditionField] != itemWriteCondition) {
                throw Error("UpdateCondition not met");
            }
            res[prop].splice(index, 1);
            res[this._lastUpdateField] = updateDate;
            return this._save(res, uid);
        });
    }
    static getModda() {
        return {
            uuid: "Webda/MemoryStore",
            label: "MemoryStore",
            description: "Implements a simple in memory store",
            webcomponents: [],
            documentation: "",
            logo: "images/icons/memorystore.png",
            configuration: {
                default: {},
                widget: {
                    tag: "webda-store-configurator",
                    url: "elements/services/webda-store-configurator.html"
                },
                schema: {
                    type: "object",
                    properties: {}
                }
            }
        };
    }
}
exports.MemoryStore = MemoryStore;
//# sourceMappingURL=memory.js.map