"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
var assert = require("assert");
const test_1 = require("../test");
const mocha_typescript_1 = require("mocha-typescript");
const Idents = require("../../test/models/ident");
class StoreTest extends test_1.WebdaTest {
    before() {
        const _super = Object.create(null, {
            before: { get: () => super.before }
        });
        return __awaiter(this, void 0, void 0, function* () {
            yield _super.before.call(this);
            yield this.getIdentStore().__clean();
            yield this.getUserStore().__clean();
        });
    }
    getModelClass() {
        return Idents;
    }
    mapper() {
        return __awaiter(this, void 0, void 0, function* () {
            let identStore = this.getIdentStore();
            let userStore = this.getUserStore();
            let user1, ident1, ident2, user2;
            var eventFired = 0;
            var events = [
                "Store.Save",
                "Store.Saved",
                "Store.Get",
                "Store.Delete",
                "Store.Deleted",
                "Store.Update",
                "Store.Updated",
                "Store.Find",
                "Store.Found"
            ];
            for (let evt in events) {
                identStore.on(events[evt], function (evt) {
                    eventFired++;
                });
            }
            assert.equal(yield userStore.get(undefined), undefined);
            user1 = (yield userStore.save({
                name: "test"
            })).uuid;
            let user = yield userStore.get(user1);
            // Save a user and add an ident
            assert.notEqual(user, undefined);
            user1 = user.uuid;
            ident1 = yield identStore.save({
                type: "facebook",
                _user: user.uuid
            });
            user = yield userStore.get(user1);
            // Verify the ident is on the user
            assert.notEqual(user, undefined);
            assert.notEqual(user.idents, undefined);
            assert.equal(user.idents.length, 1);
            // Retrieve index to verify it is in it too
            let index = yield identStore.get("index");
            assert.notEqual(index[ident1.uuid], undefined);
            assert.equal(index[ident1.uuid].type, "facebook");
            let lastUpdate = user.idents[0]._lastUpdate;
            yield this.sleep(10);
            yield identStore.incrementAttribute(ident1.uuid, "counter", 1);
            yield user.refresh();
            assert.notEqual(user.idents[0]._lastUpdate, 0);
            this.assertLastUpdateNotEqual(user.idents[0]._lastUpdate, lastUpdate, "lastUpdate on a map after incrementAttribute");
            lastUpdate = user.idents[0]._lastUpdate;
            yield this.sleep(10);
            yield identStore.upsertItemToCollection(ident1.uuid, "actions", {
                uuid: "action_1",
                type: "plop",
                date: new Date()
            });
            yield user.refresh();
            assert.notEqual(user.idents[0]._lastUpdate.length, 0);
            this.assertLastUpdateNotEqual(user.idents[0]._lastUpdate, lastUpdate, "lastUpdate on a map after upsertItemToCollection");
            lastUpdate = user.idents[0]._lastUpdate;
            yield this.sleep(10);
            yield identStore.deleteItemFromCollection(ident1.uuid, "actions", 0, "plop", "type");
            yield user.refresh();
            assert.notEqual(user.idents[0]._lastUpdate.length, 0);
            this.assertLastUpdateNotEqual(user.idents[0]._lastUpdate, lastUpdate, "lastUpdate on a map after deleteItemFromCollection");
            ident2 = yield identStore.save({
                type: "google",
                _user: user.uuid
            });
            // Add a second ident and check it is on the user aswell
            user = yield userStore.get(user1);
            assert.equal(user.idents.length, 2);
            // Check index
            yield index.refresh();
            assert.notEqual(index[ident2.uuid], undefined);
            ident2.type = "google2";
            // Update ident2 to check mapper update
            let res = yield identStore.patch({
                uuid: ident2.uuid,
                type: "google2"
            });
            assert.equal(res.type, "google2");
            assert.equal(res._user, user1);
            // Check index
            yield index.refresh();
            assert.equal(index[ident2.uuid].type, "google2");
            user = yield userStore.get(user1);
            assert.equal(user.idents.length, 2);
            assert.equal(user.idents[1].type, "google2");
            assert.equal(user.idents[1] instanceof this.getModelClass(), true);
            yield identStore.delete(ident1.uuid);
            user = yield userStore.get(user1);
            assert.equal(user.idents.length, 1);
            assert.equal(user.idents[0].type, "google2");
            // Add a second user to play
            user = yield userStore.save({
                name: "test2"
            });
            user2 = user.uuid;
            // Move ident2 from user1 to user2
            yield identStore.patch({
                _user: user.uuid,
                uuid: ident2.uuid
            });
            // Check user1 has no more ident
            user = yield userStore.get(user1);
            assert.equal(user.idents.length, 0);
            // Check user2 has one ident
            user = yield userStore.get(user2);
            assert.equal(user.idents.length, 1);
            assert.equal(user.idents[0].type, "google2");
            // Verify you cannot update a collection from update
            yield userStore.update({
                idents: []
            }, user2);
            user = yield userStore.get(user2);
            assert.equal(user.idents.length, 1);
            assert.equal(user.idents[0].type, "google2");
            // Verify delete cascade with empty collection
            yield userStore.delete(user1);
            user = yield userStore.get(user2);
            assert.equal(user.idents.length, 1);
            assert.equal(user.idents[0].type, "google2");
            // Verify delete cascade
            yield userStore.delete(user2);
            let ident = yield identStore.get(ident2.uuid);
            assert.equal(ident.__deleted, true);
            // Check index
            yield index.refresh();
            assert.equal(index[ident2.uuid], undefined);
            assert.equal(index[ident1.uuid], undefined);
            assert.equal(eventFired, 17);
            yield identStore.delete(ident2.uuid, true);
            ident = yield identStore.get(ident2.uuid);
            assert.equal(ident, undefined);
            assert.equal(eventFired, 19);
        });
    }
    collection() {
        return __awaiter(this, void 0, void 0, function* () {
            let identStore = this.getIdentStore();
            var ident;
            ident = yield identStore.save({
                test: "plop"
            });
            yield identStore.upsertItemToCollection(ident.uuid, "actions", {
                uuid: "action_1",
                type: "plop",
                date: new Date()
            });
            yield ident.refresh();
            assert.notEqual(ident.actions, undefined);
            assert.equal(ident.actions.length, 1);
            yield identStore.upsertItemToCollection(ident.uuid, "actions", {
                uuid: "action_2",
                type: "plop",
                date: new Date()
            });
            yield ident.refresh();
            assert.notEqual(ident.actions, undefined);
            assert.equal(ident.actions.length, 2);
            yield identStore.upsertItemToCollection(ident.uuid, "actions", {
                uuid: "action_1",
                type: "plop2",
                date: new Date()
            }, 0);
            yield ident.refresh();
            assert.notEqual(ident.actions, undefined);
            assert.equal(ident.actions.length, 2);
            assert.equal(ident.actions[0].type, "plop2");
            assert.equal(ident.actions[0].uuid, "action_1");
            yield this.assertThrowsAsync(identStore.upsertItemToCollection.bind(identStore, ident.uuid, "actions", {
                uuid: "action_1",
                type: "plop2",
                date: new Date()
            }, 0, "plop", "type"), err => true);
            yield this.assertThrowsAsync(identStore.deleteItemFromCollection.bind(identStore, ident.uuid, "actions", 0, "action_2"), err => true);
            yield ident.refresh();
            assert.equal(ident.actions.length, 2);
            assert.equal(ident.actions[0].type, "plop2");
            let lastUpdate = ident._lastUpdate;
            yield this.sleep(10);
            yield identStore.upsertItemToCollection(ident.uuid, "actions", {
                uuid: "action_1",
                type: "plop",
                date: new Date()
            }, 0, "plop2", "type");
            yield ident.refresh();
            this.assertLastUpdateNotEqual(ident._lastUpdate, lastUpdate, "lastUpdate after upsertItemToColletion failed");
            lastUpdate = ident._lastUpdate;
            yield this.sleep(10);
            yield identStore.deleteItemFromCollection(ident.uuid, "actions", 0, "plop", "type");
            ident = yield identStore.get(ident.uuid);
            this.assertLastUpdateNotEqual(ident._lastUpdate, lastUpdate, "lastUpdate after deleteItemToColletion failed");
            assert.notEqual(ident.actions, undefined);
            assert.equal(ident.actions.length, 1);
            assert.equal(ident.actions[0].type, "plop");
            assert.equal(ident.actions[0].uuid, "action_2");
        });
    }
    getAll() {
        return __awaiter(this, void 0, void 0, function* () {
            let userStore = this.getUserStore();
            var user1;
            var user3;
            user1 = yield userStore.save({
                name: "test1"
            });
            yield userStore.save({
                name: "test2"
            });
            user3 = yield userStore.save({
                name: "test3"
            });
            let users = yield userStore.getAll();
            assert.equal(users.length, 3);
            assert.equal(users[0] instanceof userStore._model, true);
            assert.equal(users[1] instanceof userStore._model, true);
            assert.equal(users[2] instanceof userStore._model, true);
            users = yield userStore.getAll([user1.uuid, user3.uuid]);
            assert.equal(users.length, 2);
            assert.equal(users[0] instanceof userStore._model, true);
            assert.equal(users[1] instanceof userStore._model, true);
        });
    }
    crud() {
        return __awaiter(this, void 0, void 0, function* () {
            let identStore = this.getIdentStore();
            var eventFired = 0;
            var events = [
                "Store.Save",
                "Store.Saved",
                "Store.Get",
                "Store.Delete",
                "Store.Deleted",
                "Store.Update",
                "Store.Updated",
                "Store.Find",
                "Store.Found"
            ];
            for (let evt in events) {
                identStore.on(events[evt], function (evt) {
                    eventFired++;
                });
            }
            // Check CREATE - READ
            let ident1 = yield identStore.save({
                test: "plop",
                cool: "",
                lastUsed: new Date(),
                arr: [],
                details: {
                    plop: "plop1",
                    clean: undefined,
                    yop: "pouf"
                }
            });
            assert.equal(eventFired, 2);
            assert.notEqual(ident1, undefined);
            eventFired = 0;
            let getter = yield identStore.get(ident1.uuid);
            assert.equal(eventFired, 1);
            eventFired = 0;
            assert.notEqual(getter, undefined);
            assert.notEqual(getter.lastUsed, undefined);
            assert.notEqual(getter._lastUpdate, undefined);
            assert.equal(getter.uuid, ident1.uuid);
            assert.equal(getter.test, ident1.test);
            // Check UPDATE
            getter.test = "plop2";
            getter.details.plop = "plop2";
            getter.details.blank = "";
            getter.details.bouzouf = undefined;
            getter.empty = [];
            let object;
            yield identStore.update(getter);
            assert.equal(eventFired, 2);
            eventFired = 0;
            object = yield identStore.get(ident1.uuid);
            assert.equal(object.test, "plop2");
            assert.equal(object.details.plop, "plop2");
            getter = yield identStore.get(object.uuid);
            assert.equal(eventFired, 2);
            assert.equal(getter.test, "plop2");
            yield this.sleep(10);
            yield identStore.incrementAttribute(ident1.uuid, "counter", 1);
            let ident = yield identStore.get(ident1.uuid);
            // Verify lastUpdate is updated too
            this.assertLastUpdateNotEqual(ident._lastUpdate, ident1._lastUpdate, "lastUpdate after incrementAttribute failed");
            assert.equal(ident.counter, 1);
            yield identStore.incrementAttribute(ident1.uuid, "counter", 3);
            ident1 = yield identStore.get(ident1.uuid);
            assert.equal(ident1.counter, 4);
            yield identStore.incrementAttribute(ident1.uuid, "counter", -6);
            let res = yield identStore.exists(ident1.uuid);
            assert.equal(res, true);
            ident1 = yield identStore.get(ident1.uuid);
            assert.equal(ident1.counter, -2);
            // Check DELETE
            eventFired = 0;
            yield identStore.delete(ident1.uuid, true);
            assert.equal(eventFired, 2);
            eventFired = 0;
            ident = yield identStore.get(ident1.uuid);
            assert.equal(ident, undefined);
            res = yield identStore.exists(ident1.uuid);
            assert.equal(res, false);
        });
    }
    assertLastUpdateNotEqual(d1, d2, msg) {
        assert.notEqual(d1, d2, msg);
    }
}
__decorate([
    mocha_typescript_1.test
], StoreTest.prototype, "mapper", null);
__decorate([
    mocha_typescript_1.test
], StoreTest.prototype, "collection", null);
__decorate([
    mocha_typescript_1.test
], StoreTest.prototype, "getAll", null);
__decorate([
    mocha_typescript_1.test
], StoreTest.prototype, "crud", null);
exports.StoreTest = StoreTest;
//# sourceMappingURL=store.spec.js.map