"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("./index");
const assert = require("assert");
const index_2 = require("./index");
const fs_1 = require("fs");
class WebdaTest {
    getTestConfiguration() {
        return process.cwd() + "/test/config.json";
    }
    before() {
        return __awaiter(this, void 0, void 0, function* () {
            this.webda = new index_1.Core(JSON.parse(fs_1.readFileSync(this.getTestConfiguration()).toString()));
            yield this.webda.init();
        });
    }
    newContext(body = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            let res = yield this.webda.newContext(new index_2.HttpContext("test.webda.io", "GET", "/"));
            res.getHttpContext().setBody(body);
            return res;
        });
    }
    getExecutor(ctx, host = "test.webda.io", method = "GET", url, body = {}, headers = {}) {
        ctx.setHttpContext(new index_2.HttpContext(host, method, url, "http", 80, body, headers));
        return this.webda.getExecutorWithContext(ctx);
    }
    assertThrowsAsync(fn, regExp) {
        return __awaiter(this, void 0, void 0, function* () {
            let f = () => { };
            try {
                yield fn();
            }
            catch (e) {
                f = () => {
                    throw e;
                };
            }
            finally {
                assert.throws(f, regExp);
            }
        });
    }
    sleep(time) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise(resolve => {
                setTimeout(resolve, time);
            });
        });
    }
    getService(service) {
        return this.webda.getService(service);
    }
}
exports.WebdaTest = WebdaTest;
//# sourceMappingURL=test.js.map