"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const stream_1 = require("stream");
const index_1 = require("../index");
const acceptLanguage = require("accept-language");
const cookie_1 = require("cookie");
const events_1 = require("events");
const sanitizeHtml = require("sanitize-html");
class ClientInfo extends Map {
}
exports.ClientInfo = ClientInfo;
class HttpContext {
    constructor(host, method, uri, protocol = "http", port = 80, body = {}, headers = {}, files = []) {
        this.files = files;
        this.body = body;
        this.host = host;
        this.method = method;
        this.uri = uri;
        this.protocol = protocol;
        this.port = port;
        this.headers = headers;
        for (let i in this.headers) {
            if (i.toLowerCase() === "cookie") {
                this.cookies = cookie_1.parse(this.headers[i]);
            }
            if (i.toLowerCase() !== i) {
                this.headers[i.toLowerCase()] = this.headers[i];
            }
        }
        let portUrl = "";
        if (port !== undefined && port !== 80 && protocol === "http") {
            portUrl = ":" + port;
        }
        else if (port !== undefined && port !== 443 && protocol === "https") {
            portUrl = ":" + port;
        }
        this.root = this.protocol + "://" + this.host + portUrl;
        this.origin = this.host + portUrl;
    }
    getUrl() {
        return this.uri;
    }
    getCookies() {
        return this.cookies;
    }
    getPort() {
        return this.port;
    }
    getHost() {
        return this.host;
    }
    getMethod() {
        return this.method;
    }
    getProtocol() {
        return this.protocol;
    }
    getBody() {
        return this.body;
    }
    getHeaders() {
        return this.headers;
    }
    getHeader(name) {
        return this.headers[name.toLowerCase()];
    }
    setBody(body) {
        this.body = body;
    }
    getFullUrl(uri = this.uri) {
        return this.protocol + "://" + this.host + uri;
    }
}
exports.HttpContext = HttpContext;
class Cookie {
}
class Context extends events_1.EventEmitter {
    /**
     * @ignore
     * Used by Webda framework to set the body, session and output stream if known
     */
    constructor(webda, httpContext, stream = undefined) {
        super();
        this._ended = undefined;
        this._params = undefined;
        this.clientInfo = new ClientInfo();
        this._http = httpContext;
        this._webda = webda;
        this._promises = [];
        this._outputHeaders = new Map();
        this._flushHeaders = false;
        this._body = undefined;
        this.statusCode = 204;
        this._stream = stream;
        this._buffered = false;
        this._params = {};
        this.headers = new Map();
        if (stream === undefined) {
            this._stream = new stream_1.Writable();
            this._stream._body = [];
            this._stream._write = this._write;
        }
        this.processParameters();
        this.session = this.newSession();
    }
    /**
     * @private
     * Used in case of Buffer response ( like Lambda )
     */
    _write(chunk, enc, next) {
        if (this._body === undefined) {
            this._body = [];
        }
        this._body.push(chunk);
        next();
        return true;
    }
    /**
     * Set current http context
     * @param httpContext current http context
     */
    setHttpContext(httpContext) {
        this._http = httpContext;
        this.reinit();
    }
    reinit() {
        this._sanitized = undefined;
    }
    /**
     * Get current http context
     */
    getHttpContext() {
        return this._http;
    }
    /**
     * Get output headers
     */
    getResponseHeaders() {
        return this._outputHeaders;
    }
    getRequestParameters() {
        return this._params;
    }
    parameter(name) {
        return this.getParameters()[name];
    }
    getParameters() {
        return this._params;
    }
    processParameters() {
        this._params = Object.assign({}, this._serviceParams);
        this._params = Object.assign(this._params, this._pathParams);
    }
    getSession() {
        return this.session.getProxy();
    }
    getServiceParameters() {
        return this._serviceParams;
    }
    getPathParameters() {
        return this._pathParams;
    }
    setServiceParameters(params) {
        this._serviceParams = params;
        this.processParameters();
    }
    setPathParameters(params) {
        this._pathParams = params;
        this.processParameters();
    }
    resetResponse() {
        this._body = undefined;
        this._outputHeaders.clear();
    }
    /**
     * Write data to the client
     *
     * @param output If it is an object it will be serializeb with toPublicJSON, if it is a String it will be appended to the result, if it is a buffer it will replace the result
     * @param ...args any arguments to pass to the toPublicJSON method
     */
    // @ts-ignore
    write(output, encoding, cb) {
        if (typeof output === "object" && !(output instanceof Buffer)) {
            this._outputHeaders["Content-type"] = "application/json";
            // @ts-ignore
            global.WebdaContext = this;
            try {
                this._body = JSON.stringify(output);
            }
            finally {
                // @ts-ignore
                global.WebdaContext = undefined;
            }
            return true;
        }
        else if (typeof output == "string") {
            if (this._body == undefined) {
                this._body = "";
            }
            this._body += output;
            return true;
        }
        else {
            this._body = output;
        }
    }
    /**
     * Set a header value
     *
     * @param {String} header name
     * @param {String} value
     */
    setHeader(header, value) {
        this._outputHeaders[header] = value;
    }
    /**
     * Write the http return code and some headers
     * Those headers are not flushed yet so can still be overwritten
     *
     * @param {Number} httpCode to return to the client
     * @param {Object} headers to add to the response
     */
    writeHead(httpCode, headers) {
        index_1._extend(this._outputHeaders, headers);
        if (httpCode !== undefined) {
            this.statusCode = httpCode;
        }
    }
    /**
     * Redirect to another url
     * @param url
     */
    redirect(url) {
        this.writeHead(302, { Location: url });
    }
    /**
     * For compatibility reason
     *
     * @todo Implement the serialization
     * Not yet handle by the Webda framework
     */
    cookie(param, value, options) {
        /** @ignore */
        if (this._cookie === undefined) {
            this._cookie = new Map();
        }
        this._cookie[param] = { name: param, value, options };
    }
    getResponseCookies() {
        return this._cookie;
    }
    addAsyncRequest(promise) {
        this._promises.push(promise);
    }
    isEnded() {
        return this._ended;
    }
    /**
     * Flush the request
     *
     * @emits 'finish' event
     * @throws Error if the request was already ended
     */
    end() {
        return __awaiter(this, void 0, void 0, function* () {
            /** @ignore */
            if (this._ended) {
                return this._ended;
            }
            this._ended = new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                this.emit("end");
                yield Promise.all(this._promises);
                if (this._buffered && this._stream._body !== undefined) {
                    this._body = Buffer.concat(this._stream._body);
                }
                if (!this._flushHeaders) {
                    this._flushHeaders = true;
                    if (this._body !== undefined && this.statusCode == 204) {
                        this.statusCode = 200;
                    }
                    this._webda.flushHeaders(this);
                }
                this._webda.flush(this);
                this.emit("close");
                resolve();
            }));
            return this._ended;
        });
    }
    getRequestBody(sanitizedOptions = {
        allowedTags: [],
        allowedAttributes: {}
    }) {
        if (this._sanitized) {
            return this._sanitized;
        }
        let recursiveSanitize = (obj, options = undefined) => {
            if (!obj) {
                return obj;
            }
            if (typeof obj === "string") {
                return sanitizeHtml(obj, options);
            }
            if (typeof obj === "object") {
                Object.keys(obj).forEach(key => {
                    obj[key] = recursiveSanitize(obj[key], options);
                });
            }
            return obj;
        };
        this._sanitized = recursiveSanitize(this.getHttpContext().getBody(), sanitizedOptions);
        return this._sanitized;
    }
    getResponseBody() {
        return this._body;
    }
    getRequest() {
        const s = require("stream");
        var stream = new s.Readable();
        stream.push(JSON.stringify(this.getRequestBody()));
        return Object.assign({ httpVersionMajor: 1, httpVersionMinor: 0, headers: this.headers, httpVersion: "1.0", method: this.getHttpContext().getMethod(), rawHeaders: [], rawTrailers: [], setTimeout: (msec, callback) => { }, socket: undefined, statusCode: 200, trailers: {}, url: this.getHttpContext().getUrl(), connection: undefined }, stream);
    }
    /**
     * Get a service from webda
     *
     * @see Webda
     * @param {String} name of the service
     */
    getService(name) {
        return this._webda.getService(name);
    }
    /**
     * Get a service from webda
     *
     * @see Webda
     * @param {String} name of the service
     */
    getTypedService(name) {
        return this._webda.getService(name);
    }
    /**
     * Get the HTTP stream to output raw data
     * @returns {*}
     */
    getStream() {
        return this._stream;
    }
    /**
     * Get the current user from session
     */
    getCurrentUser() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.getCurrentUserId()) {
                return undefined;
            }
            return this._webda.getService("Users").get(this.getCurrentUserId());
        });
    }
    /**
     * Get the current user id from session
     */
    getCurrentUserId() {
        if (this.session) {
            return this.session.getUserId();
        }
        return undefined;
    }
    /**
     * Return the webda
     */
    getWebda() {
        return this._webda;
    }
    /**
     * Get the request locale if found
     */
    getLocale() {
        let locales = this._webda.getLocales();
        acceptLanguage.languages(locales);
        let headers = this.getHttpContext().getHeaders();
        if (headers["Accept-Language"]) {
            return acceptLanguage.get(headers["Accept-Language"]);
        }
        return locales[0];
    }
    /**
     * @ignore
     * Used by Webda framework to set the current route
     */
    setRoute(route) {
        this._route = route;
        this._params = index_1._extend(this._params, route.params);
    }
    getRoute() {
        return this._route;
    }
    /**
     * @param executor {object} Set the current executor for this context
     */
    setExecutor(executor) {
        this._executor = executor;
    }
    /**
     * @ignore
     * Used for compatibility with express module
     */
    logIn() { }
    newSession() {
        this.session = new (this._webda.getModel(this._webda.parameter("sessionModel") || "WebdaCore/SessionCookie"))(this);
        return this.session;
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            this._stream.on("pipe", () => {
                this._flushHeaders = true;
                this._buffered = true;
                this._webda.flushHeaders(this);
            });
            yield this.session.init();
        });
    }
    emitError(err) {
        this.emit("error", err);
    }
}
exports.Context = Context;
class StoreSessionContext extends Context {
}
class CookieSessionContext extends Context {
}
//# sourceMappingURL=context.js.map